@@ -1,6 +1,6 @@
 """Library for converting Swagger objects to Python data types such as dataframes."""
 import re
-from typing import Any, Dict, Union
+from typing import Any, Dict, List, Union
 
 import numpy as np
 import pandas as pd
@@ -11,11 +11,14 @@ from rime_sdk.swagger.swagger_client.models import (
     TestrunresultTestCase,
     TestrunresultTestRunDetail,
 )
+from rime_sdk.swagger.swagger_client.models.detection_detection_event import (
+    DetectionDetectionEvent,
+)
 
 MODEL_PERF_REGEX = r"^metrics\.model_perf.*(ref_metric|eval_metric)$"
 
 # Map of flattened field paths to their types in the Dataframe.
-DEFAULT_TEST_RUN_COLUMN_INFO = {
+TEST_RUN_COLUMN_TO_SHOW = {
     # Metadata.
     "test_run_id": "str",
     "name": "str",
@@ -23,14 +26,8 @@ DEFAULT_TEST_RUN_COLUMN_INFO = {
     "model_task": "str",  # OPTIONAL
     # The canonical JSON encoding converts enums to their string representations
     # so we do not need to do manual conversions.
-    "data_type": "str",
     "testing_type": "str",
     "upload_time": "str",
-    # Model source info.
-    "model_source_info.name": "str",  # OPTIONAL
-    # Data source info.
-    "data_source_info.ref.name": "str",
-    "data_source_info.eval.name": "str",
     # Metrics.
     "metrics.duration_millis": "Int64",
     "metrics.num_inputs": "Int64",
@@ -42,7 +39,6 @@ DEFAULT_TEST_RUN_COLUMN_INFO = {
     "metrics.summary_counts.skip": "Int64",
     "metrics.severity_counts.num_none_severity": "Int64",
     "metrics.severity_counts.num_low_severity": "Int64",
-    "metrics.severity_counts.num_medium_severity": "Int64",
     "metrics.severity_counts.num_high_severity": "Int64",
 }
 
@@ -51,7 +47,6 @@ SUMMARY_TEST_KEYS_TO_HIDE = [
     "category_metrics",
     "description",
     "duration",
-    "id",
     "suggestion",
 ]
 
@@ -62,10 +57,10 @@ TEST_BATCH_KEYS_TO_HIDE = [
     "display",
 ]
 
-# List of all the columns to hide for a test case DF.
+# List of all the columns to hide for a Test Case DF.
 TEST_CASE_KEYS_TO_HIDE = [
     "display",
-    "feature_id",
+    "url_safe_feature_id",
     "test_case_id",
 ]
 
@@ -92,6 +87,28 @@ DATA_OPTIONS = [
     "str_list",
 ]
 
+EVENT_COLUMNS_TO_SHOW = [
+    "project_id",
+    "event_type",
+    "severity",
+    "event_object_name",
+    "event_object_id",
+    "risk_category_type",
+    "test_category",
+    "event_time_range.start_time",
+    "event_time_range.end_time",
+    "description",
+    "detail.metric_degradation.rca_result.description",
+]
+
+
+def _flatten_uuid_field_name(field_name: str) -> str:
+    """Flatten a UUID field name."""
+    match = re.match(r"(.*)\.uuid$", field_name)
+    if match is None:
+        return field_name
+    return match.groups()[0]
+
 
 def parse_test_run_metadata(test_run: TestrunresultTestRunDetail) -> pd.DataFrame:
     """Parse test run metadata Swagger message into a Pandas dataframe.
@@ -106,8 +123,9 @@ def parse_test_run_metadata(test_run: TestrunresultTestRunDetail) -> pd.DataFram
     # Flatten out nested fields in the Protobuf message.
     # The DF column name will be the field path joined by the `df_flatten_separator.`
     normalized_df = pd.json_normalize(test_run_dict, sep=DF_FLATTEN_SEPARATOR)
+    normalized_df = normalized_df.rename(_flatten_uuid_field_name, axis="columns")
 
-    default_test_run_columns = list(DEFAULT_TEST_RUN_COLUMN_INFO.keys())
+    default_test_run_columns = list(TEST_RUN_COLUMN_TO_SHOW.keys())
 
     # Include the model perf columns with the set of DF columns.
     # These are metrics like "Accuracy" over the reference and eval datasets.
@@ -131,13 +149,13 @@ def parse_test_run_metadata(test_run: TestrunresultTestRunDetail) -> pd.DataFram
     # https://developers.google.com/protocol-buffers/docs/proto3#json
     # Note the type of all model perf metrics should be float64 so we do not have
     # to do this conversion.
-    for key, value in DEFAULT_TEST_RUN_COLUMN_INFO.items():
+    for key, value in TEST_RUN_COLUMN_TO_SHOW.items():
         if value == "Int64":
             # Some nested fields such as `metrics.severity_counts.low` will be `None`
             # because MessageToDict does not populate nested primitive fields with
             # default values.
             # Since some columns may be `None`, we must convert to `float` first.
-            # https://stackoverflow.com/questions/60024262/error-converting-object-string-to-int32-typeerror-object-cannot-be-converted  # pylint: disable=line-too-long
+            # https://stackoverflow.com/questions/60024262/error-converting-object-string-to-int32-typeerror-object-cannot-be-converted
             full_df[key] = full_df[key].astype("float").astype("Int64")
 
     # Fix an order for the index of the df.
@@ -149,7 +167,8 @@ def parse_test_run_metadata(test_run: TestrunresultTestRunDetail) -> pd.DataFram
 
 
 def parse_test_batch_result(
-    raw_result: TestrunresultTestBatchResult, unpack_metrics: bool = False,
+    raw_result: TestrunresultTestBatchResult,
+    unpack_metrics: bool = False,
 ) -> pd.Series:
     """Parse test batch result into a series."""
     result_dict = raw_result.to_dict()
@@ -169,7 +188,7 @@ def parse_test_batch_result(
         # because MessageToDict does not populate nested primitive fields with
         # default values.
         # Since some columns may be `None`, we must convert to `float` first.
-        # https://stackoverflow.com/questions/60024262/error-converting-object-string-to-int32-typeerror-object-cannot-be-converted  # pylint: disable=line-too-long
+        # https://stackoverflow.com/questions/60024262/error-converting-object-string-to-int32-typeerror-object-cannot-be-converted
         df[key] = df[key].astype("float").astype("Int64")
 
     return df.squeeze(axis=0)
@@ -178,7 +197,7 @@ def parse_test_batch_result(
 def parse_test_case_result(
     raw_result: TestrunresultTestCase, unpack_metrics: bool = False
 ) -> dict:
-    """Parse swagger test case result to pythonic form."""
+    """Parse swagger Test Case result to pythonic form."""
     result_dict = raw_result.to_dict()
     del result_dict["metrics"]
     if unpack_metrics:
@@ -198,7 +217,7 @@ def _add_metric_cols(
     raw_result: Union[TestrunresultTestCase, TestrunresultTestBatchResult],
 ) -> None:
     """Unpack test metrics into separate fields."""
-    if raw_result.metrics:  # pylint: disable=too-many-nested-blocks
+    if raw_result.metrics:
         for metric in raw_result.metrics:
             category_string = metric.category
             if category_string:
@@ -216,8 +235,9 @@ def _add_metric_cols(
                 result_dict[f"{category_string}:{metric.metric}"] = metric_value
 
 
-def parse_summary_test_result(
-    raw_result: RimeCategoryTestResult, unpack_metrics: bool = False,
+def parse_category_test_results(
+    raw_result: RimeCategoryTestResult,
+    unpack_metrics: bool = False,
 ) -> dict:
     """Parse swagger summary test result to pythonic form."""
     raw_dict = raw_result.to_dict()
@@ -230,3 +250,17 @@ def parse_summary_test_result(
     for key in severity_count_dict:
         raw_dict[key] = int(severity_count_dict[key])
     return raw_dict
+
+
+def parse_events_to_df(
+    events: List[DetectionDetectionEvent],
+) -> pd.DataFrame:
+    """Parse a list of Detection Events to a pandas DateFrame."""
+    event_dicts = [event.to_dict() for event in events]
+    df = pd.json_normalize(event_dicts, sep=DF_FLATTEN_SEPARATOR)
+    df = df.rename(_flatten_uuid_field_name, axis="columns")
+
+    intersect_df = df[df.columns.intersection(EVENT_COLUMNS_TO_SHOW)]
+    intersect_df = intersect_df.rename({"event_object_id": "monitor_id"}, axis=1)
+
+    return intersect_df
