@@ -5,17 +5,14 @@ import time
 from abc import ABC, abstractmethod
 from datetime import datetime
 from http import HTTPStatus
-from typing import Any, Dict, Optional
-
-import pandas as pd
-from deprecated import deprecated
+from typing import Any, Dict, List, Optional
 
 from rime_sdk.internal.rest_error_handler import RESTErrorHandler
 from rime_sdk.swagger import swagger_client
 from rime_sdk.swagger.swagger_client import ApiClient
+from rime_sdk.swagger.swagger_client.models import RimeJobData, RimeJobMetadata
+from rime_sdk.swagger.swagger_client.models import RimeJobStatus as StatedbJobStatus
 from rime_sdk.swagger.swagger_client.models import (
-    RimeJobMetadata,
-    RimeJobStatus,
     RimeJobType,
     RimeJobView,
     RimeTestRunProgress,
@@ -24,15 +21,15 @@ from rime_sdk.swagger.swagger_client.models import (
 from rime_sdk.swagger.swagger_client.rest import ApiException
 from rime_sdk.test_run import TestRun
 
-default_csv_header = ["test_name", "feature(s)", "status"]
-cancellable_job_types = [
+CANCELLABLE_JOB_TYPES = [
+    RimeJobType.FILE_SCAN,
     RimeJobType.FIREWALL_BATCH_TEST,
     RimeJobType.MODEL_STRESS_TEST,
 ]
 # Format datetimes in user-readable format
-datetime_fields = ["creation_time", "start_time"]
+DATETIME_FIELDS = ["creation_time", "start_time", "completion_time"]
 # Removed if empty or overly verbose
-fields_to_be_removed = ["job_data", "archived_job_logs"]
+FIELDS_TO_BE_REMOVED = ["job_data", "archived_job_logs"]
 
 
 class BaseJob(ABC):
@@ -49,9 +46,9 @@ class BaseJob(ABC):
 
         Args:
             api_client: ApiClient
-                The client used to query about the status of the job.
+                The client used to query the RIME cluster.
             job_id: str
-                The identifier for the RIME job that this object monitors.
+                The ID of the RIME Job object.
         """
         self._api_client = api_client
         self._job_id = job_id
@@ -62,27 +59,38 @@ class BaseJob(ABC):
 
     @property
     def job_id(self) -> str:
-        """Return the id of the job."""
+        """Return the ID of the Job."""
         return self._job_id
 
     @property
     def job_type(self) -> str:
-        """Return the type of the job."""
+        """Return the type of the Job.
+
+        The valid Job types are:
+        MODEL_STRESS_TEST, FIREWALL_BATCH_TEST, IMAGE_BUILDER, FILE_SCAN
+        """
         return self._job_type
 
+    @property
+    def error_msg(self) -> str:
+        """The error message, if this job failed."""
+        status = self.get_status(verbose=False)
+        return f"{status.get('error_msg')}"
+
     def __str__(self) -> str:
         """Pretty-print the object."""
-        # Ping the backend for the status of the job and detailed info only for
-        # succeeded jobs.
-        status = self.get_status()
+        # Ping the backend for the full representation of the job and detailed
+        # info only for succeeded jobs.
+        job = self.get_status(verbose=False)
         ret = {
             "job_id": self._job_id,
             "job_type": self._job_type,
-            "status": status.get("status"),
-            "termination_reason": status.get("terminationReason"),
+            "status": job.get("status"),
         }
-        if status.get("status") == RimeJobStatus.SUCCEEDED:
+        if job.get("status") == StatedbJobStatus.SUCCEEDED:
             ret.update(self._get_test_run_dict())
+        elif job.get("status") == StatedbJobStatus.FAILED:
+            ret["error_msg"] = job.get("error_msg")
         job_class = self.__class__.__name__
         return f"{job_class} {ret}"
 
@@ -96,86 +104,84 @@ class BaseJob(ABC):
         if len(test_batches) == 0:
             return None
         n = sum(batch.status == RimeTestTaskStatus.COMPLETED for batch in test_batches)
-        return "{:<2} / {:>2} tests completed".format(n, len(test_batches))
+        return f"{n:<2} / {len(test_batches):>2} tests completed"
 
     @abstractmethod
     def _get_progress(self, job: RimeJobMetadata) -> Optional[str]:
         """Pretty print the progress of the test run."""
 
-    def get_status(
+    # TODO(VAL-2282): refactor this function to have fewer branches and be more readable
+    def get_status(  # noqa: PLR0912 (too-many-branches)
         self,
-        verbose: bool = False,
+        verbose: bool = True,
         wait_until_finish: bool = False,
         poll_rate_sec: float = 5.0,
     ) -> Dict:
-        """Query the ModelTest service for the job's status.
+        """Return the current status of the Job.
 
-        This includes flags for blocking until the job is complete and printing
-        information to ``stdout``. This method can help with monitoring the
-        progress of stress test jobs, because it prints out helpful information
-        such as running time and the progress of the test run.
+        Includes flags for blocking until the job is complete and printing
+        information to ``stdout``. This method displays the progress and
+        running time of jobs.
 
         If the job has failed, the logs of the testing engine will be dumped
-        to ``stdout`` to help with debuggability.
+        to ``stdout`` for debugging.
 
         Arguments:
             verbose: bool
-                whether or not to print diagnostic information such as progress.
-                Note that these logs have no strict form and will be subject to
-                significant change in future versions.
+                Specifies whether to print diagnostic information such as progress.
             wait_until_finish: bool
-                whether or not to block until the job has succeeded or failed.
+                Specifies whether to block until the job has succeeded or failed.
                 If `verbose` is enabled too, information about the job including
                 running time and progress will be printed to ``stdout`` every
                 ``poll_rate_sec``.
             poll_rate_sec: float
-               the frequency with which to poll the job's status. Units are in seconds.
+                The frequency in seconds with which to poll the job's status.
 
         Returns:
-            A dictionary representing the job's state.
+            Dict:
+                A dictionary representing the job's state.
 
-            .. code-block:: python
+                .. code-block:: python
 
-                {
-                "id": str
-                "type": str
-                "status": str
-                "start_time_secs": int64
-                "running_time_secs": double
-                }
+                    {
+                    "id": str
+                    "type": str
+                    "status": str
+                    "start_time_secs": int64
+                    "running_time_secs": double
+                    }
 
         Example:
+            .. code-block:: python
 
-        .. code-block:: python
-
-            # Block until this job is finished and dump monitoring info to stdout.
-            job_status = job.get_status(verbose=True, wait_until_finish=True)
+                # Block until this job is finished and dump monitoring info to stdout.
+                job_status = job.get_status(verbose=True, wait_until_finish=True)
         """
         printed_cancellation_req = False
+        printed_poll_string = False
         # Create backend client stubs to use for the remainder of this session.
         api = swagger_client.JobReaderApi(self._api_client)
         with RESTErrorHandler():
-            job: RimeJobMetadata = api.job_reader_get_job(
+            job: RimeJobMetadata = api.get_job(
                 job_id=self._job_id, view=RimeJobView.FULL
             ).job
         if verbose:
             print(
                 "Job '{}' started at {}".format(
-                    job.job_id, datetime.fromtimestamp(job.creation_time.timestamp()),
+                    job.job_id,
+                    datetime.fromtimestamp(job.creation_time.timestamp()),
                 )
             )
 
         # Do not repeat if the job is finished or blocking is disabled.
         while wait_until_finish and job.status not in (
-            RimeJobStatus.SUCCEEDED,
-            RimeJobStatus.FAILED,
-            RimeJobStatus.CANCELLED,
+            StatedbJobStatus.SUCCEEDED,
+            StatedbJobStatus.FAILED,
+            StatedbJobStatus.CANCELLED,
         ):
             time.sleep(poll_rate_sec)
             with RESTErrorHandler():
-                job = api.job_reader_get_job(
-                    job_id=self._job_id, view=RimeJobView.FULL
-                ).job
+                job = api.get_job(job_id=self._job_id, view=RimeJobView.FULL).job
                 progress = self._get_progress(job)
             if verbose:
                 if (
@@ -189,22 +195,29 @@ class BaseJob(ABC):
 
                 minute, second = divmod(job.running_time_secs, 60)
                 hour, minute = divmod(minute, 60)
-                progress_str = " ({})".format(progress) if progress else ""
+                progress_str = f" ({progress})" if progress else ""
                 print(
                     "\rStatus: {}, Running Time: {:02}:{:02}:{:05.2f}{}".format(
                         job.status, int(hour), int(minute), second, progress_str
                     ),
                     end="",
                 )
+                printed_poll_string = True
+
+        # Add a new line if we printed the poll string, because it has `end=""`.
+        # This ensures future print statements start on the next line.
+        if verbose and printed_poll_string:
+            print()
 
         # Only get the logs if the job has failed, as the
         # primary purpose is debuggability during development.
         model_testing_api = swagger_client.ModelTestingApi(self._api_client)
-        if job.status == RimeJobStatus.FAILED:
+        if job.status == StatedbJobStatus.FAILED:
+            if job.error_msg is not None:
+                print(f"Error message from failed job: {job.error_msg}")
+
             try:
-                response = model_testing_api.model_testing_get_latest_logs(
-                    job_id=self._job_id
-                )
+                response = model_testing_api.get_latest_logs(job_id=self._job_id)
                 if response.result.chunk:
                     print(
                         "\nLogs from failed job:\n"
@@ -213,18 +226,20 @@ class BaseJob(ABC):
                 else:
                     print("No logs found.")
             except ApiException as e:
-                if e.status == HTTPStatus.NOT_FOUND:
-                    print("Unable to retrieve logs for failed job.")
-                else:
+                # if logs cannot be found do not raise an error.
+                # For most deployments logs cannot be read via this api.
+                if e.status != HTTPStatus.NOT_FOUND:
                     raise ValueError(e.reason) from None
+            # check for debug logs in the job
+            print(f"Debug Logs: {self._get_debug_logs(job)}")
 
         job_dict = job.to_dict()
         # Hide job data and archived logs in the return value because it can get ugly.
-        for field in fields_to_be_removed:
+        for field in FIELDS_TO_BE_REMOVED:
             if field in job_dict:
                 del job_dict[field]
 
-        for field in datetime_fields:
+        for field in DATETIME_FIELDS:
             if field in job_dict and job_dict[field]:
                 job_dict[field] = job_dict[field].strftime("%H:%M %B %d, %Y")
 
@@ -241,52 +256,91 @@ class BaseJob(ABC):
         return {}
 
     def cancel(self) -> None:
-        """Request that the job be cancelled in the backend.
+        """Request to cancel the Job.
 
-        The backend will mark the job with "Cancellation Requested" and then clean
-        up the job in the background.
+        The RIME cluster will mark the Job with "Cancellation Requested" and then clean
+        up the Job.
         """
-        if self.job_type not in cancellable_job_types:
+        if self.job_type not in CANCELLABLE_JOB_TYPES:
             raise ValueError(
-                (
-                    f"Cancelling jobs is only supported for "
-                    f"job types {cancellable_job_types}"
-                )
+                "Cancelling jobs is only supported for job types "
+                f"{CANCELLABLE_JOB_TYPES}"
             )
         api = swagger_client.JobReaderApi(self._api_client)
         with RESTErrorHandler():
-            api.job_reader_cancel_job(job_id=self.job_id)
+            api.cancel_job(job_id=self.job_id)
 
     def get_agent_id(self) -> str:
-        """Get the agent ID which is running a successful job."""
+        """Return the Agent ID running the Job."""
         api = swagger_client.JobReaderApi(self._api_client)
         with RESTErrorHandler():
-            job = api.job_reader_get_job(job_id=self._job_id).job
-        return job.agent_id
+            job = api.get_job(job_id=self._job_id).job
+        return job.agent_id.uuid
+
+    def _get_job(self) -> RimeJobData:
+        """Return the Job object.
+
+        Get the Test Run ID corresponding to a successful Job.
+
+        Raises:
+            ValueError
+                This error is generated when the job does not have state 'SUCCEEDED'
+                or when the request to the Job Reader service fails.
+        """
+        # This first step only prevents a rare case where the RIME engine has
+        # signaled the test suite has completed but before the upload has completed.
+        api = swagger_client.JobReaderApi(self._api_client)
+        with RESTErrorHandler():
+            job: RimeJobMetadata = api.get_job(job_id=self._job_id).job
+        if job.status != StatedbJobStatus.SUCCEEDED:
+            raise ValueError(
+                f"Job has status {job.status}; "
+                "it must have status {StatedbJobStatus.SUCCEEDED} to get results"
+            )
+        return job.job_data
 
     def get_job_debug_logs_link(self) -> str:
-        """Get the archived logs (as a link) for failed job."""
+        """Return a link to the archived logs for a failed Job."""
         api = swagger_client.JobReaderApi(self._api_client)
         with RESTErrorHandler():
-            job: RimeJobMetadata = api.job_reader_get_job(
+            job: RimeJobMetadata = api.get_job(
                 job_id=self._job_id, view=RimeJobView.FULL
             ).job
-            if job.status != RimeJobStatus.FAILED:
-                raise ValueError(
-                    "debug logs are only available for "
-                    "failed jobs, this job has status {}".format(job.status)
-                )
-            else:
+            return self._get_debug_logs(job)
+
+    def _get_debug_logs(self, job: RimeJobMetadata) -> str:
+        """Pretty print the archived logs for a failed Job."""
+        if job.status != StatedbJobStatus.FAILED:
+            return (
+                f"Debug logs are not available for this job."
+                f"They are only available for failed jobs, "
+                f"this job has status {job.status}"
+            )
+        else:
+            try:
                 expiration_time = datetime.fromtimestamp(
                     job.archived_job_logs.expiration_time.timestamp()
                 )
-                if (
-                    job.archived_job_logs.url.url != ""
-                    and datetime.now() < expiration_time
-                ):
-                    return job.archived_job_logs.url.url
+                if datetime.now() < expiration_time:
+                    return (
+                        f"Debug logs for the job can found in your blob "
+                        f"storage for your job {job.job_id},"
+                        f" at the following url {job.archived_job_logs.url}"
+                    )
                 else:
-                    raise ValueError("debug logs are not available for this job")
+                    return (
+                        f"Debug logs for the job {job.job_id} maybe "
+                        f"available in your blob storage. RI platform only"
+                        f" has access to a presigned url which has "
+                        f"expired. However, the logs themselves might not "
+                        f"have expired. Please check your configured blob"
+                        f" storage. The logs maybe available in the "
+                        f"bucket configured under the following directory:"
+                        f" /logs/{job.job_id}. The expired url:"
+                        f"{job.archived_job_logs.url}"
+                    )
+            except AttributeError:
+                return "Debug logs are not available for this job"
 
 
 class Job(BaseJob):
@@ -302,25 +356,23 @@ class Job(BaseJob):
             return self._get_test_run_progress(job.job_data.stress.progress.test_run)
 
     def get_test_run_id(self) -> str:
-        """Get the test run ID for a successful job."""
-        # This first step only prevents a rare case where the RIME engine has
-        # signaled the test suite has completed but before the upload has completed.
-        api = swagger_client.JobReaderApi(self._api_client)
-        with RESTErrorHandler():
-            job: RimeJobMetadata = api.job_reader_get_job(job_id=self._job_id).job
-        if job.status != RimeJobStatus.SUCCEEDED:
-            raise ValueError(
-                "Job has status {}; it must have status {} to get results".format(
-                    job.status, RimeJobStatus.SUCCEEDED
-                )
-            )
-        return job.job_data.stress.test_run_id
-
-    def get_test_run(self) -> TestRun:
-        """Get the test run object.
+        """Get the Test Run ID corresponding to a successful Job.
 
         Raises:
-            ValueError if the job does not have state 'SUCCEEDED.'
+            ValueError
+                This error is generated when the job does not have state 'SUCCEEDED'
+                or when the request to the Job Reader service fails.
+        """
+        job_data = self._get_job()
+        return job_data.stress.test_run_id
+
+    def get_test_run(self) -> TestRun:
+        """Get the Test Run object corresponding to a successful Job.
+
+        Raises:
+            ValueError
+                This error is generated when the job does not have state 'SUCCEEDED'
+                or if the request to the Job Reader service fails.
         """
         test_run_id = self.get_test_run_id()
         return TestRun(self._api_client, test_run_id)
@@ -336,95 +388,6 @@ class Job(BaseJob):
                 ret[k] = v
         return ret
 
-    @deprecated("Use TestRun.get_link()")
-    def get_link(self) -> str:  # noqa: D401
-        """Deprecated.
-
-        Get the web app URL for a successful stress test job.
-
-        This link directs to your organization's deployment of RIME.
-        You can view more detailed information about the results of your stress test\
-        in the web app, including helpful visualiziations, key insights, and\
-        explanations of test results.
-
-        Note: this is a string that should be copy-pasted into a browser.
-
-        :meta private:
-        """
-        test_run = self.get_test_run()
-        return test_run.get_link()
-
-    @deprecated("Use TestRun.get_test_run_result()")
-    def get_test_run_result(self) -> pd.DataFrame:  # noqa: D401
-        """Deprecated.
-
-        Retrieve high level summary information.
-        This dataframe includes information such as model metrics on the reference and\
-        evaluation datasets, overall RIME results such as severity across tests,\
-        and high level metadata such as the project ID and model task.
-        By concatenating these rows together, this allows you to build a table of test
-        run results for sake of comparison. This only works on stress test jobs that
-        have succeeded.
-        Note: this does not work on <0.14.0 RIME test runs.
-
-        Returns:
-            A `pandas.DataFrame` object containing the test run result.
-            There are a lot of columns, so it is worth viewing them with the `.columns`
-            method to see what they are. Generally, these columns have information
-            about the model and datasets as well as summary statistics like the number
-            of failing test cases or number of high severity test cases.
-
-        Example:
-        .. code-block:: python
-            # Wait until the job has finished, since this method only works on
-            # succeeded jobs.
-            job.get_status(verbose=True, wait_until_finish=True)
-            # Dump the test cases in dataframe ``df``.
-            # Print out the column names and types.
-            print(df.columns)
-
-        :meta private:
-        """
-        test_run_obj = self.get_test_run()
-        return test_run_obj.get_result_df()
-
-    @deprecated("Use TestRun.get_test_cases_result()")
-    def get_test_cases_result(self) -> pd.DataFrame:  # noqa: D401
-        """Deprecated.
-
-        Retrieve all the test cases in a dataframe.
-        This gives you the ability to perform granular queries on
-        test cases. For example, if you only care about subset performance tests and
-        want to see the results on each feature, you can fetch all the test cases in
-        a dataframe, then query on that dataframe by test type. This only works on
-        stress test jobs that have succeeded.
-        Note: this will not work for test runs run on RIME versions <0.14.0.
-
-        Returns:
-            A ``pandas.DataFrame`` object containing the test case results.
-            Here is a selected list of columns in the output:
-            1. ``test_run_id``: ID of the parent test run.
-            2. ``features``: List of features that the test case ran on.
-            3. ``test_batch_type``: Type of test that was run (e.g. Subset AUC,\
-                Must be Int, etc.).
-            4. ``status``: Status of the test case (e.g. Pass, Fail, Skip, etc.).
-            5. ``severity``: Metric that denotes the severity of the failure of\
-                the test.
-
-        Example:
-        .. code-block:: python
-            # Wait until the job has finished, since this method only works on
-            # SUCCEEDED jobs.
-            job.get_status(verbose=True, wait_until_finish=True)
-            # Dump the test cases in dataframe ``df``.
-            # Print out the column names and types.
-            print(df.columns)
-
-        :meta private:
-        """
-        test_run_obj = self.get_test_run()
-        return test_run_obj.get_test_cases_df()
-
 
 class ContinuousTestJob(BaseJob):
     """This object provides an interface for monitoring a Continuous Test Job in the RIME backend."""
@@ -456,6 +419,28 @@ class ContinuousTestJob(BaseJob):
                     return f"{run_progress}, {batch_progress}"
         return run_progress
 
+    def get_test_run_ids(self) -> str:
+        """Get the set of Test Run IDs corresponding to a successful Continuous Test Job.
+
+        Raises:
+            ValueError
+                This error is generated when the job does not have state 'SUCCEEDED'
+                or when the request to the Job Reader service fails.
+        """
+        job_data = self._get_job()
+        return job_data.continuous_inc.ct_test_run_ids
+
+    def get_test_runs(self) -> List[TestRun]:
+        """Get the list of Test Run objects corresponding to a successful Continuous Test Job.
+
+        Raises:
+            ValueError
+                This error is generated when the job does not have state 'SUCCEEDED'
+                or if the request to the Job Reader service fails.
+        """
+        test_run_ids = self.get_test_run_ids()
+        return [TestRun(self._api_client, test_run_id) for test_run_id in test_run_ids]
+
     def _get_test_run_dict(self) -> dict:
         """Return a dictionary that is used to help pretty-print this object."""
         return {}
@@ -466,7 +451,9 @@ class ImageBuilderJob(BaseJob):
 
     _job_type = RimeJobType.IMAGE_BUILDER
 
-    def _get_progress(self, job: RimeJobMetadata) -> Optional[str]:
+    def _get_progress(
+        self, job: RimeJobMetadata  # noqa: ARG002 (unused-method-argument)
+    ) -> Optional[str]:
         """Pretty print the progress of the test run."""
         # TODO: find a good way to pretty print the progress of a ImageBuilderJob
         return None
@@ -477,7 +464,21 @@ class FileScanJob(BaseJob):
 
     _job_type = RimeJobType.FILE_SCAN
 
-    def _get_progress(self, job: RimeJobMetadata) -> Optional[str]:
+    def _get_progress(
+        self, job: RimeJobMetadata  # noqa: ARG002 (unused-method-argument)
+    ) -> Optional[str]:
         """Pretty print the progress of the test run."""
         # TODO: find a good way to pretty print the progress of a FileScanJob
         return None
+
+
+class GenerativeModelTestJob(BaseJob):
+    """This object provides an interface for monitoring a Generative Model Test Job in the RIME backend."""
+
+    _job_type = RimeJobType.GENERATIVE_MODEL_TEST
+
+    def _get_progress(
+        self, job: RimeJobMetadata  # noqa: ARG002 (unused-method-argument)
+    ) -> Optional[str]:
+        """Pretty print the progress of the test run."""
+        return None
